"""
Done
Implemented cd
Implemented pwd
Implemented ls
Implemented cat
Implemented find -size
Implemented find -type
Implemented find -name with regex support
Implemented grep

Todo
Implement grep *
"""
#-----------------
import sourcestats
import statnode
import utils
#-----------------
import sys
import cmd
import argparse
import re
import subprocess
import pygments
import pygments.lexers
import pygments.formatters
#-----------------


class CodeBrowser(cmd.Cmd):
    def __init__(self,stat):
        cmd.Cmd.__init__(self)
        self.prompt       = "> "
        self.stat         = stat.stats
        self.current_node = self.stat
        self.last_node    = self.stat

    def change_node(self,new_node):
        self.last_node    = self.current_node
        self.current_node = new_node
        
    @utils.show_problem
    def do_grep(self,line=""):
        #------------------------------------------------------------------
        args      = line.split(" ")        
        argparser = argparse.ArgumentParser()
        argparser.add_argument("pattern",help="pattern",type=str,nargs="?")
        argparser.add_argument("target" ,help="target" ,type=str,nargs="?")
        #------------------------------------------------------------------
        if not(args and args[0]):
            return
        args  = argparser.parse_args(args)
        lines = []        
        for node in self.walk():
            utils.debug("[DEBUG do_grep] walking down nodes : ",node.path,"looking for",args.pattern)
            for line,start,end in self._test_grep(node,args.pattern):
                lines.append((node.path,utils.colorize(line,start,end)))
        if lines:
            utils.align(lines,":")

        print "%s matches" % len(lines)
        
    def do_pwd(self,line=""):
        print self.current_node.path

    def do_less(self,line=""):
        node = self.current_node.get_node(line)
        if not node:
            node = self.current_node
        less = subprocess.Popen(["less",'-R'],stdin=subprocess.PIPE)
        code = "\n".join(node.get_code())
        formatted_source = pygments.highlight(code,pygments.lexers.PythonLexer(),pygments.formatters.TerminalFormatter(colorscheme=utils.CAT_COLORS,bg="dark"))
        less.stdin.write(formatted_source)
        less.stdin.close()
        less.wait()

    def do_symbols(self,line=""):
        symbols      = [node.name for node in self.walk()]
        for nuple in utils.columnize(symbols):
            print " ".join(name for name in nuple if name)
        print "%s symbols defined" % len(symbols)

    @utils.show_problem
    def complete_jump(self,*args):
        """
        Prints all defined functions and classes.
        """
        incomplete_filename = args[0]
        L                   = [node.name for node in self.walk(self.stat) if node.name.startswith(incomplete_filename)]
        return L

    complete_j = complete_jump

    @utils.show_problem
    def complete_cd(self,*args):
        user_input = args[1]
        arg        = user_input.split(" ")[1]
        last_arg   = arg.split("/")[-1]
        if not last_arg:
            last_node  = arg
        else:
            last_node = arg.split(last_arg)[0]
        node = self.current_node.get_node(last_node)
        if not node:
            node = self.current_node
        all_children = reduce(lambda x,y:x+y,[subnode.keys() for subnode in node.get_catalog()])
        children     = [child for child in all_children if child.startswith(last_arg)]
        if len(children) == 1:
            children[0] = children[0] + "/"
        return children

    complete_ls = complete_cat = complete_less = complete_cd
    
    def do_jump(self,line=""):
        for node in self.walk(self.stat):
            if node.name == line:
                self.change_node(node)
                break
        self.prompt = self.current_node.path + "> "

    do_j = do_jump
        
    @utils.hide_problem
    def do_ls(self,line=""):
        #------------------------------------------------------------------
        args        = line.split(" ")        
        argparser   = argparse.ArgumentParser()
        argparser.add_argument("target"         ,help="target"                                ,type = str    , nargs="?"                        )        
        argparser.add_argument("-s","--sort"    ,help='sort by size or by name'               ,type = str                                       )
        argparser.add_argument("-S"             ,help='sort by size'                          ,dest = "sort" , action="store_const",const="size") 
        argparser.add_argument("-r","--reverse" ,help="reverse sort order, implies -S/--sort"                , action="store_true"              )        

        if (args and args[0]):
            args    = argparser.parse_args(args)
            target  = args.target
            sort    = args.sort
            reverse = args.reverse
        else:
            target  = ""
            sort    = None
            reverse = None

        #------------------------------------------------------------------
        node = self.current_node.get_node(target)
        if not node : 
            node = self.current_node
        node.ls(sort=sort,reverse=reverse)

    def do_cd(self,line=""):
        if not line :
            self.change_node(self.stat)
        elif line == "..":
            self.change_node(self.current_node.parent)
        elif line == "-":
            self.change_node(self.last_node)
        else : 
            node = self.current_node.get_node(line)
            if node :
                self.change_node(node)
        self.prompt = self.current_node.path+"> "
        
    @utils.show_problem
    def do_cat(self,line=""):
        #------------------------------------------------------------------
        args        = line.split(" ")        
        argparser   = argparse.ArgumentParser()
        colors_list = "autumn,borland,bw,colorful,default,emacs,friendly,fruity,manni,murphy,native,pastie,perldoc,trac,vim"        
        argparser.add_argument("target"       ,help="target"                                                                 ,type = str,nargs="?")
        argparser.add_argument("-s","--style" ,help="style names as defined in pygments. Hardcoded values = %s" % colors_list,type = str          )        
        args  = argparser.parse_args(args)
        #------------------------------------------------------------------        
        node  = self.current_node.get_node(args.target)
        utils.debug("[DEBUG do_cat] %s.cat()" % node.path)
        if not node :
            node = self.current_node
        node.cat(args.style)
            
    @utils.hide_problem
    def do_find(self,line):
        nodes         = []
        starting_node = self.current_node
        parser        = argparse.ArgumentParser()
        # ---------------------------------------
        parser.add_argument("starting_node"  ,help="find nodes starting from here"                   ,type   = str, nargs="?", default=".")
        parser.add_argument("-s","--size"    ,help="size +/- X lines of code"                        ,type   = int)
        parser.add_argument("-S","--sort"    ,help="sort by loc, sort by size (alias), sort by name" ,type   = str)
        parser.add_argument("-r","--reverse" ,help="reverse sort order, implies -S/--sort"           ,action = "store_true")                
        parser.add_argument("-t","--type"    ,help="-t (method | function | class | module)"         ,type   = str)
        parser.add_argument("-n","--name"    ,help="search by name"                                  ,type   = str)
        # ---------------------------------------
        args = line.split(" ")
        if args and args[0]:
            args         = parser.parse_args(args)
            starting_node = self.current_node.get_node(args.starting_node)
        nodes = self.select_nodes(starting_node,
                                  node_size=getattr(args,"size",None),
                                  node_type=getattr(args,"type",None),
                                  node_name=getattr(args,"name",None),
                                  )
        keyfunc = None
        if getattr(args,"sort"):
            sort = args.sort
            if sort.replace(" ","") in "bysize,byloc,loc,size".split(","):
                keyfunc = lambda x:x.loc
            elif sort.replace(" ","") in "byname,name".split(","):
                keyfunc = lambda x:x.name
        if keyfunc:
            nodes.sort(key=keyfunc,reverse=getattr(args,"reverse",False) == True)

        self.print_nodes(nodes)
        print "total matches :",len(nodes)

    def select_nodes(self,starting_node,**options):
        """
        """
        nodes = []
        for node in self.walk(starting_node):
            # -----------------------------------------------------------------------------------
            if options.get("node_type") and not self._test_type(node,options.get("node_type")):
                continue
            if options.get("node_size") and not self._test_size(node,options.get("node_size")):
                continue
            if options.get("node_name") and not self._test_name(node,options.get("node_name")):
                continue
            # -----------------------------------------------------------------------------------            
            nodes.append(node)
                
        return nodes

    def _test_name(self,node,node_name):
        """
        """
        return re.match(node_name,node.name)
        
    def _test_type(self,node,node_type):
        """
        """
        return node.node_type == node_type
        

    def _test_size(self,node,size):
        return (size > 0 and node.loc >= size) or (node.loc <= -size)

    def _test_grep(self,node,pattern):
        if not hasattr(getattr(node,'code',object),"source"):
            return
        code = node.code.source
        for line in code.split("\n"):
            match = re.search(pattern,line)
            if match :
                yield line,match.start(),match.end()
    
    def print_nodes(self,nodes):
        """
        Used by do_find
        """
        current_path = self.current_node.path
        for node in nodes:
            print "%s:%s" % (node.path.split(current_path)[1].lstrip("/"),node.loc)            

    def is_accountable(self,node):
        """
        """
        #return node.node_type in ["method","function","module","class","code"]
        #return not isinstance(node,statnode.PackageStatNode)
        return True


    def walk(self,node=None):
        if not node:
            node = self.current_node

        # node can be a dict or a CodeStatNode
        for subnode in node.get_catalog():
            for child in subnode.values():
                if self.is_accountable(child):
                    yield child
                for grandchild in self.walk(child):
                    yield grandchild
                    

    def do_quit(self,line):
        print '<Quit>'
        return True

    # Aliases
    do_q    = do_quit
    do_exit = do_quit

def main():
    import readline
    # this caused me a lot of trouble and I couldn't figure out why.
    # so I had to remove the dash from this list to be able to complete
    # names that included dashes.
    readline.set_completer_delims(' \t\n`~!@#$%^&*()=+[{]}\\|;:/\'",<>?')
    if len(sys.argv) < 2 :
        usage()
        exit(1)
    st = sourcestats.SourceStats(sys.argv[1:])
    CodeBrowser(st).cmdloop()
    
def usage():
    print sys.argv[0],"file_path"

if __name__ == "__main__":
    main()
