"""
Needs a good docstring template here.
"""

#----------------
# Local imports
#----------------
import utils
import statnode
import reader
#----------------
# Standard Python Library imports
#----------------
import tokenize
import sys,os
import dircache
#----------------

class SourceStats:
    def __init__(self,file_paths):
        # ----------------------------------------------------------------------------------------------------------------
        self.stats = statnode.PackageStatNode(name="root",parent=None)
        # ----------------------------------------------------------------------------------------------------------------
        self._move_cursor_to(self.stats)
        if type(file_paths) == str :
            file_paths = [file_paths]
        # ----------------------------------------------------------------------------------------------------------------
        for file_path in file_paths:
            utils.debug("[DEBUG SourceStats.__init__] file_path",file_path)
            self.process_input_files(self.stats,file_path)
        # ----------------------------------------------------------------------------------------------------------------

    def process_input_files(self,current_node,file_path):
        if not os.access(file_path,os.R_OK):
            utils.debug("[DEBUG SourceStats.process_input_files] we don't have read permission on",file_path)
            return
        node_name = utils.get_python_module_name(file_path)
        # ---------------------------------------------------------------------------------------------
        if os.path.isdir(file_path):
            utils.debug("[DEBUG process_input_files] processing",node_name,"package")
            current_node.packages[node_name] = statnode.PackageStatNode(name=node_name,parent=current_node)
            for filename in dircache.listdir(file_path):
                complete_file_name = os.path.join(file_path,filename)
                self.process_input_files(current_node.packages[node_name],complete_file_name)
        # ---------------------------------------------------------------------------------------------
        elif file_path.endswith(".py"):
            utils.debug("[DEBUG process_input_files] processing",node_name,"module")
            current_node.modules[node_name] = statnode.ModuleStatNode( name      = node_name,
                                                                     parent    = current_node,
                                                                     depth     = -1,
                                                                     node_type = "module"
                                                                    )
            code = open(file_path).readlines()
            self._move_cursor_to(current_node.modules[node_name])
            self.process_python_code(code)
            self._move_cursor_to(current_node)


    def process_python_code(self,code):
        reader.Reader(self).process(code)

    #  +----------------------------------------+
    #  |                                        |
    #  | These methods are called by the Reader |
    #  |                                        |
    #  +----------------------------------------+

    def register_class(self,name,depth):
        utils.debug("[DEBUG register_class] current node is",self.cursor.path)
        utils.debug("[DEBUG register_class] recording class",name,"at depth",depth)
        utils.debug("[DEBUG register_class] current depth is",self.cursor.depth)
        self.cursor.classes[name] = statnode.ModuleStatNode(name=name,parent=self.cursor,depth=depth,node_type="class")
        self._move_cursor_to(self.cursor.classes[name])
        # self.cursor.code.source += line
        utils.debug("[DEBUG register_class] registered class",self.cursor.path)

    def register_def(self,name,depth):
        utils.debug("[DEBUG register_def] register %s with depth %s" %(name,depth))
        utils.debug("[DEBUG register_def] current node is",self.cursor.path,"with depth",self.cursor.depth)
        self.cursor.functions[name] = statnode.ModuleStatNode( name      = name,
                                                               parent    = self.cursor,
                                                               node_type = "function",
                                                               depth     = depth
                                                             )
        self._move_cursor_to(self.cursor.functions[name])
        # self.cursor.code.source += line
        utils.debug("[DEBUG register_def] registered %s function at %s" % (self.cursor.name,self.cursor.path))

    def register_loc(self,line,incr=True):
        utils.debug("[DEBUG SourceStats.register_loc] '%s'" % line)
        utils.debug("[DEBUG SourceStats.register_loc] increment is",incr)
        target_node       = self.cursor.code
        target_node_path  = target_node.path
        current_node_path = self.cursor.path
        utils.debug("[DEBUG SourceStats.register_loc] target_node  is %s " % target_node_path)
        utils.debug("[DEBUG SourceStats.register_loc] current_node is %s " % current_node_path)
        target_node.source += line
        if incr:
            target_node.incr_stat()
        utils.debug("[DEBUG SourceStats.register_loc] target's type %s " % target_node.node_type)
        ## if target_node.parent.node_type == "module":
        ##     self._move_cursor_to(target_node)


    def change_depth(self,new_depth):
        utils.debug("[SourceStats.change_depth] new depth %s" % new_depth)
        utils.debug("[SourceStats.change_depth] current depth %s" % self.cursor.depth)
        while new_depth <= self.cursor.depth: # at parent level
            utils.debug("[SourceStats.change_depth] up one parent")
            utils.debug("[SourceStats.change_depth] from %s" % self.cursor.path)
            self._move_cursor_to(self.cursor.parent)
            utils.debug("[SourceStats.change_depth] to %s" % self.cursor.path)

    #  +----------------------------------------+
    #  |                                        |
    #  | These methods are called internally    |
    #  |                                        |
    #  +----------------------------------------+

    def _move_cursor_to(self,node):
        utils.debug("[DEBUG _move_cursor_to]",node.path)
        self.cursor = node

def usage():
    print "no no no. Use %s like this : python %s path_to_file.py" % (sys.argv[0],sys.argv[0])

def main():
    if len(sys.argv) < 2 :
        usage()
        exit(1)
    st = SourceStats(sys.argv[1:])

if __name__ == "__main__":
    main()

