import optparse
import sys
import ConfigParser
import functools
import os
import traceback
import pygments.token
import itertools
import commands
"""
dark_colors  = ["black", "darkred", "darkgreen", "brown", "darkblue",
                "purple", "teal", "lightgray"]
light_colors = ["darkgray", "red", "green", "yellow", "blue",
                "fuchsia", "turquoise", "white"]
"""
CAT_COLORS = {
    pygments.token.Token              : ('**'          , '**'         ),
    pygments.token.Whitespace         : ('lightgray'   , 'darkgray'   ),
    pygments.token.Comment	          : ('lightgray'   , 'red'        ),
    pygments.token.Comment.Preproc    : ('teal'        , 'darkgrey'   ),
    pygments.token.Keyword	          : ('darkblue'    , 'green'      ),
    pygments.token.Keyword.Type       : ('teal'	       , 'purple'     ),
    pygments.token.Operator.Word      : ('purple'	   , 'white'      ),
    pygments.token.Name.Builtin       : ('teal'	       , 'green'      ),
    pygments.token.Name.Function      : ('darkgreen'   , '*yellow*'   ),
    pygments.token.Name.Namespace     : ('_teal_'	   , 'brown'      ),
    pygments.token.Name.Class	      : ('_darkgreen_' , '_blue_'     ),
    pygments.token.Name.Exception     : ('teal'	       , '*red*'      ),
    pygments.token.Name.Decorator     : ('darkgray'	   , '*turquoise*'),
    pygments.token.Name.Variable      : ('darkred'	   , 'white'      ),
    pygments.token.Name.Constant      : ('darkred'	   , '*lightgray*'),
    pygments.token.Name.Attribute     : ('teal'	       , 'white'      ),
    pygments.token.Name.Tag	          : ('blue'	       , ''           ),
    pygments.token.String	          : ('brown'       , 'fuchsia'    ),
    pygments.token.Number	          : ('darkblue'    , '*white*'    ),
    #-------------------------------------------------------------------
    pygments.token.Generic.Deleted    : ('red'         , 'red'        ),
    pygments.token.Generic.Inserted   : ('darkgreen'   , 'green'      ),
    pygments.token.Generic.Heading    : ('**'          , '**'         ),
    pygments.token.Generic.Subheading : ('*purple*'    , '*fuchsia*'  ),
    pygments.token.Generic.Error      : ('red'         , 'red'        ),
    #-------------------------------------------------------------------
    pygments.token.Error	          : ('_red_'       , '_red_'      ),
    }



def show_problem(function):
    def wrapper(*args,**kw): 
        try:
            return function(*args,**kw)
        except:
            print traceback.print_exc()
    return wrapper

def hide_problem(function):
    def wrapper(*args,**kw): 
        try:
            return function(*args,**kw)
        except:
            pass
    return wrapper

def get_python_module_name(file_path):
    return os.path.basename(file_path.strip("/")).split(".py")[0]

def header(text):
    print "-"*48
    print text
    print "-"*48    

def align(strings_list,separator=":"):
    if not strings_list:
        return
    length = max([len(string[0]) for string in strings_list])
    for string in strings_list:
        print string[0].ljust(length),separator,string[1]

def real_debug(*args,**kw):
    # Allow aything but filter out this
    filter_out = kw.get("filter_out")
    # Allow nothing that doesn't pass this filter
    filter_in= kw.get("filter_in")
    if filter_out :
        filter_out_debug(filter_out,*args)
    elif filter_in :
        filter_in_debug(filter_in,*args)
    else: 
        just_debug(*args)

def filter_in_debug(filter_in,*args):
    for string in filter_in:
        if string in args[0]:
            just_debug(*args)
            return
    
def filter_out_debug(filter_out,*args):
    for string in filter_out: 
        if string in args[0]:
            return
    just_debug(*args)
            
def just_debug(*args):
    for arg in args[0:-1]:
        print arg,
    print args[-1]
    
parser = optparse.OptionParser()
parser.add_option("-d","--debug",help="debug",action="store_true")
parser.add_option("-c","--config_file",help="configuration file for the debugging utility")
options,args = parser.parse_args(sys.argv)
parser = ConfigParser.ConfigParser()
filepath = options.config_file
if filepath : 
    parser.read(filepath)
    
filter_in  = None
filter_out = None

if parser.has_option("default","filter_in"): 
    filter_in  = parser.get("default","filter_in").split(",")
    
elif parser.has_option("default","filter_out"):
    filter_out = parser.get("default","filter_out").split(",")

if options.debug:
    if filter_out :
        debug = functools.partial(filter_out_debug,filter_out)
    elif filter_in :
        debug = functools.partial(filter_in_debug,filter_in)
    else: 
        debug = just_debug
else:
    debug = lambda *x:None

    
def colorize(line,start,end):
    start_code = '\033[1;4m'
    end_code   = '\033[m'
    return line[:start] + start_code + line[start:end] + end_code + line[end:]

def _columnize(names,ncolumns):
    """
    iterator
    """
    out     = []
    columns = {}
    for i in xrange(ncolumns):
        columns[i]               = {"names":[],"max_length":0,}
        # first pass
        columns[i]["names"]      = [name.strip() for name in names[i::ncolumns]]
        columns[i]["max_length"] = len(max(columns[i]["names"],key= lambda x:len(x)))
        # second pass
        columns[i]["names"]      = [name.ljust(columns[i]["max_length"]) for name in columns[i]["names"]]
    
    return itertools.izip_longest(*(columns[i]["names"] for i in xrange(ncolumns)))

def columnize(words,ncolumns = None):
    # maximum terminal width is 149 characters
    if ncolumns:
        return _columnize(words,ncolumns)
    _words     = list(words)
    max_width  = 149
    space_left = max_width
    ncolumns   = 0
    while space_left > 1 and _words:
        longest     = max(_words,key = lambda x:len(x))        
        ncolumns    += 1        
        space_left -= len(longest)+1
        _words.remove(longest)
    return _columnize(words,ncolumns)
    
